

uniform sampler2D bgl_RenderedTexture;

const float BLOOM_THRESHOLD = 0.7;
const float BLOOM_INTENSITY = 3.0;
const int BLUR_ITERATIONS = 3;
const int BLUR_SUBDIVISIONS = 32;
const float BLUR_SIZE = 0.03;


vec3 getHDR(vec3 tex)
{
	return max((tex - BLOOM_THRESHOLD) * BLOOM_INTENSITY, 0.0);
}

vec3 gaussian(sampler2D sampler, vec2 uv)
{
	vec3 sum = vec3(0.0);

	for(int i=1; i <= BLUR_ITERATIONS; i++)
	{

		float angle = 360. / float(BLUR_SUBDIVISIONS);

		for(int j=0; j < BLUR_SUBDIVISIONS; j++)
		{
			float dist = BLUR_SIZE * (float(i + 1) / float(BLUR_ITERATIONS));
			float s = sin(angle * float(j));
			float c = cos(angle * float(j));

			sum += getHDR(texture2D(sampler, uv + vec2(c,s)*dist).xyz);
		}

	}

	sum /= float(BLUR_ITERATIONS * BLUR_SUBDIVISIONS);

	return sum * BLOOM_INTENSITY;
}

vec3 blend(vec3 a, vec3 b)
{
	return 1.0 - (1.0 - a) * (1.0 - b);
}

void main()
{
	vec2 uv = gl_TexCoord[0].st;

	vec4 tx = texture2D(bgl_RenderedTexture, uv);

	gl_FragColor.rgb = gaussian(bgl_RenderedTexture, uv);
	gl_FragColor.a= tx.a;
	gl_FragColor.xyz = blend(tx.rgb, gl_FragColor.rgb);
}